/*
   Classes to construct and display directed graphs (with cycles).

   This code is a part of Model EyeQ (www.modeleyeq.com), a documentation
   tool for Rational Rose models.
   Copyright 1999 Synergex
*/


/////////////////////////////////////////////////////////////////////
// GRAPH
/////////////////////////////////////////////////////////////////////

/* Class Graph: Represents a directed graph that may have cycles. */
function Graph()
{
    this.nodeSequenceNum = 0;  // A sequence number for nodes that provides unique IDs.
    this.root = null;                       // The root node of this graph.
    this.visited = [];                      // An array of booleans that records whether we've visited a node.

    this.setRoot = setRoot;
    this.createNode = createNode;
    this.computeSpanningTree = computeSpanningTree;
    this.computeSpanningTree2 = computeSpanningTree2;
}

/* Method of Graph: Set the root node of a graph. */
function setRoot(node)
{
    this.root = node;
}

/* Method of Graph: Create a new node with a given label. */
function createNode(label, url)
{
    var ID = this.nodeSequenceNum++;
    return new Node(ID, label, url);
}

/* Method of Graph: Determine a depth-first spanning tree for a graph. */
function computeSpanningTree()
{
    this.visited = new Array(this.nodeSequenceNum);
    for (var i = 0; i < this.nodeSequenceNum; i++)
    {
        this.visited[i] = false;
    }

    if (this.root != null)
    {
        this.visited[this.root.ID] = true;
        this.computeSpanningTree2(this.root);
    }
}

/* Method of Graph: The recursive part of computeSpanningTree() that actually does the work. */
function computeSpanningTree2(node)
{
    for (var i = 0; i < node.successors.length; i++)
    {
        var successor = node.successors[i];
        if (this.visited[successor.ID])
        {
            node.edgeStatus[i] = false;
        }
        else
        {
            this.visited[successor.ID] = true;
            node.edgeStatus[i] = true;
            this.computeSpanningTree2(successor);
        }
    }
}


/////////////////////////////////////////////////////////////////////
// NODE
/////////////////////////////////////////////////////////////////////

/* Class Node: Represents a node in a directed graph. */
function Node(ID, label, url)
{
    this.ID = ID;               // An integer that uniquely identifies a node in a given graph.
    this.label = label;       // A string that gives a (possibly non-unique) label for this node.
    this.url = url;             // A URL to link to from this node.
    this.successors = [];  // Contains nodes that have incoming edges from this node.
    this.edgeStatus = [];   // An array of boolean values that parallels {successors} and records
                                    // whether an edge is part of a depth-first spanning tree.

    this.addSuccessor = addSuccessor;
}

/* Method of Node: Add a successor to a node. */
function addSuccessor(node)
{
    // The Array object in JScript doesn't have the push() method.
    //this.successors.push(node);
    this.successors[this.successors.length] = node;
}


/////////////////////////////////////////////////////////////////////
// GRAPHPRINTER
/////////////////////////////////////////////////////////////////////

/* Class GraphPrinter: A pretty-printer of graphs. */
function GraphPrinter()
{
    this.indentation = [];  // An array of booleans that records information about 
                                    // whether there should be a "|" at a particular indentation level.

    this.printGraph = printGraph;
    this.printNode = printNode;
    this.printIndentation = printIndentation;
}

/* Method of GraphPrinter: Prints the leading indentation of a node. */
function printIndentation(depth)
{
    for (var i = 0; i < depth; i++)
    {
        if (this.indentation[i])
        {
            document.write("|  ");
        }
        else
        {
            document.write("   ");
        }
    }

    document.write("+--");
}

/* Method of GraphPrinter: Prints a graph. */
function printGraph(graph)
{
    graph.computeSpanningTree();
    if (graph.root != null)
    {
        this.indentation[0] = false;
        this.printNode(graph.root, 1);
    }
}

/* Method of GraphPrinter: Prints a node. */
function printNode(node, depth)
{
    this.printIndentation(depth);
    printLabel(node.label, node.url);
    for (var i = 0; i < node.successors.length; i++)
    {
        if (i < node.successors.length - 1)
        {
            this.indentation[depth + 1] = true;
        }
        else
        {
            this.indentation[depth + 1] = false;
        }

        var successor = node.successors[i];
        
        if (node.edgeStatus[i])
        {
            this.printNode(successor, depth + 1);
        }
        else
        {
            this.printIndentation(depth + 1);
            document.write("@");
            printLabel(successor.label.italics(), successor.url);
        }
    }

    this.indentation[depth + 1] = false;  // Not necessary, but nice to tidy up.
}

function printLabel(label, url)
{
    if (url != "")
    {
        document.writeln("<A HREF='" + url + "'>" + label + "</A>");
    }
    else
    {
        document.writeln(label);
    }
}
